unit janRunButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,shellApi,
  ExtCtrls, inifiles;


type
  TjanRunButton = class(TGraphicControl)
  private
    FPushDown: boolean;
    FMouseOver:boolean;
    FColor: TColor;
    FGlyph: TBitmap;
    FFileName: String;
    FFlat: boolean;
    procedure SetFileName(const Value: String);
    procedure SetFlat(const Value: boolean);
    { Private declarations }
  protected
    { Protected declarations }
    procedure CMMouseLeave(var Message: TMessage); message CM_MouseLeave;
    procedure CMMouseEnter(var Message: TMessage); message CM_MouseEnter;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Loaded; override;
  public
    {With TjanRunButton you allow the user to right-click select a program file.}
    {}
    {The Icon of the selected program is displayed on the button.}
    {}
    {When the user left-clicks the button the program is launched.}
    {}
    {The programname auto-persists in the application inifile.}
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    {Creates and initializes an instance of TjanRunButton. }
    destructor Destroy; override;
    {Destroys an instance of TjanRunButton.}
    procedure Paint; override;
    {Renders the image of the button.}
  published
    { Published declarations }
    property Flat:boolean read FFlat write SetFlat;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Jans 2', [TjanRunButton]);
end;

{ TjanRunButton }



constructor TjanRunButton.Create(AOwner: TComponent);
begin
  inherited;
  width := 35;
  height := 35;
  FColor := clSilver;
  FPushDown := false;
  FGlyph:=TBitmap.create;
  FGlyph.width:=32;
  FGlyph.height:=32;
  FFlat:=true;
  showhint:=true;
end;

procedure TjanRunButton.MouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if button=mbleft then
  begin
    FPushDown := true;
    invalidate;
  end;
end;

procedure TjanRunButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var od:TOpenDialog;
begin
  if button=mbleft then
  begin
    FPushDown := false;
    invalidate;
    if FileExists(FFilename) then
      shellexecute(application.handle,'open',@FFilename[1],nil,nil,SW_SHOWNORMAL);
  end
  else if button=mbright then
  begin
    od:=Topendialog.create(self);
    if od.Execute then SetFileName(od.filename);
    od.free;
  end;
end;

procedure TjanRunButton.Paint;
var R:TRect;

  procedure drawGlyph;
  begin
    if FGlyph<>nil then
    begin
      FGlyph.transparent:=true;
      canvas.Draw (1,1,FGlyph);
    end;
  end;

  procedure drawbackground(AColor: TColor);
  begin
      canvas.brush.color := AColor;
      canvas.FillRect(R);
  end;

  procedure drawup;
  begin
    drawbackground(FColor);
    Frame3D( Self.Canvas, R, clBtnHighlight, clBlack, 1 );
    drawGlyph;
  end;

  procedure drawflat;
  begin
    drawbackground(FColor);
    drawGlyph
  end;

  procedure drawdown;
  begin
    drawbackground(FColor);
    Frame3D( Self.Canvas, R, clblack, clBtnHighlight, 1 );
    drawGlyph;
  end;



begin
  R:=rect(0,0,width,height);
  if (csDesigning in ComponentState) then drawup
  else if FPushDown then drawdown
  else if not FFlat then drawup
  else if FMouseover then drawup
  else drawflat;
end;


destructor TjanRunButton.Destroy;
var ini:TIniFile;
    ifn:string;
    pfn:string;
begin
  ifn:=changefileext(application.exename,'.ini.');
  ini:=TIniFile.Create (ifn);
  ini.WriteString('RunButtons',self.name,FFilename);
  ini.free;
  FGlyph.free;
  inherited;
end;

procedure TjanRunButton.SetFileName(const Value: String);
var AIcon:TIcon;
    PC: Array[0..$FF] of Char;
begin
  FFileName := Value;
  if fileexists(FFilename) then
  begin
    AIcon:=TIcon.create;
    StrPCopy(PC, FFileName);
    AIcon.Handle := ExtractIcon(0, PC, 0);
    FGlyph.canvas.draw(0, 0, AIcon);
    AIcon.free;
    Hint:=changefileext(extractfilename(FFileName),'');
  end
  else
  begin
    FFilename:='';
    Hint:='';
    FGlyph:=nil;
  end;
  invalidate;
end;

procedure TjanRunButton.Loaded;
var ini:TIniFile;
    ifn:string;
    pfn:string;
begin
  inherited;
  ifn:=changefileext(application.exename,'.ini.');
  ini:=TIniFile.Create (ifn);
  pfn:=ini.ReadString('RunButtons',self.name,'');
  ini.free;
  if fileexists(pfn) then
    setfilename(pfn);
end;

procedure TjanRunButton.SetFlat(const Value: boolean);
begin
  FFlat := Value;
end;

procedure TjanRunButton.CMMouseEnter(var Message: TMessage);
begin
  FMouseOver:=true;
  invalidate;
end;

procedure TjanRunButton.CMMouseLeave(var Message: TMessage);
begin
  FMouseOver:=false;
  invalidate;
end;

end.

